{{
toggle.spin
Propgcc - PASM toggle demo

Simple PASM routine to demonstrate interaction between PASM subroutines and a PROPGCC main program.
The code running in the C address space to exchange data values with PASM code running
in an independent COG.

The C program has visibility/access to the mailbox variables as normal C variables.
The PASM program has visibility/access to the mailbox variables through the PAR register
initialized by the COGNEW and hte RDLONG/RDWORD/RDCHAR and WRLONG/WRWORD/WRCHAR instructions.  

C program starts the PASM routine via cognew() function, passing start address and PAR register value.
PAR register should be the address of a STATIC VOLATILE data area of LONGs in the C program.

For this example:

PAR ->  static volatile unsigned int pins;
        static volatile unsigned int wait_time;

The two C variables are used as input to the PASM routine. The C program can change these after it starts the PASM
code but the PASM code will only see their values when it reads them from HUB memory. If the program doesn't read them again after
initially starts, the changes made  by the C code will never be seen.

This could have been written as more efficient PASM code but for the examples, I was going for maximum clarity at this point.

Copyright (c) 2011, Steve Denson, Rick Post
MIT Licensed. Terms of use below.

}}

pub start(pinptr)                                                  
    cognew(@pasm, pinptr)

dat             org 0

pasm
                mov      mailbox_ptr,par         ' save the pointer to the STATIC parameter (HUB) memory area
                                                 ' the PAR register is initialized by the cognew() function and is a pointer to
                                                 ' the first STATIC int declared in the C code as the mailbox
                                                 ' mailbox_ptr will be changed as the code executes. You can reload
                                                 ' the initial pointer from PAR if you ever need it to point to
                                                 ' the start of the mailbox again
                rdlong   pins, mailbox_ptr       ' read the pin mask from HUB - it is initialized by the C program
                                                 ' in the C program, this is initially set as $3FFFFFFF
                add      mailbox_ptr,#4          ' point to the next LONG in HUB
                mov      dira, pins              ' set pins as provided by C program to OUTPUT
:loop           rdlong   waitdelay, mailbox_ptr  ' get the waitdelay from HUB in case the C program changed it
                mov      nextcnt, waitdelay      ' calculate the wait
                add      nextcnt, cnt            ' best to add cnt last
                xor      outa, pins              ' toggle pins
                waitcnt  nextcnt, waitdelay      ' wait for user specified delay
                jmp      #:loop                  ' loop until the C provided counter hits zero


' these do not need to be in any particular order or have particular names. There is no relationship between these
' local copies of the C variable except when you create via the PAR register and HUB instructions
' there is no address resolution or linkage done by propgcc or the loader
'
mailbox_ptr     long    0                       ' working ptr into the HUB area - reload from PAR if needed
pins            long    0                       ' local copy of the user's PIN mask
waitdelay       long    0                       ' local copy of the user's delay
nextcnt         long    0                       '
{{

MIT Licensed.

+--------------------------------------------------------------------
 TERMS OF USE: MIT License
+--------------------------------------------------------------------
Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files
(the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+--------------------------------------------------------------------
}}